
intrinsic ParametrizeAnticanonicalP1xP1(I::RngMPol: 
  precomputedLie := false, pLrep := 0, pL := 0, 
  Diag := false, statisticsFile := "") -> SeqEnum
{ find a parametrization of anticanonically embedded P1xP1,
  given by a homogeneous ideal}

  require (#Basis(I) gt 0): "Nontrivial ideal expected.";
  R := Parent(Basis(I)[1]);
  require (CoefficientRing(R) eq Rationals()): "A variety defined over rationals expected.";
  require (IsHomogeneous(I)): 
    "A projective variety should be defined by a homogeneous ideal.";
  require (Rank(R) eq 9): "A variety in 8-dimensional projective space expected.";
  for i := 1 to #Basis(I) do
    require (TotalDegree(Basis(I)[i]) eq 2): "A variety defined by quadratic forms expected.";
  end for;

  if not precomputedLie then

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "FindLieAlgebra time:");
      t := Cputime();
    end if;
    //  ######  end write statistics  ######

    print "Computing the Lie algebra";
    Lrep, L := FindLieAlgebra(I);
    if (Dimension(L) ne 6) then
      print "The variety is not anticanonically embedded P1xP1";
      print "  - the Lie algebra does not have the correct dimension";
      return [ ];
    end if;

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, Cputime(t));

      cf_max := 0;
      for i := 1 to 6 do
        for j := 1 to 6 do
          if (i ne j) then
            cf := Eltseq(L ! Basis(L)[i]*Basis(L)[j]);
            for j := 1 to #cf do
              cf[j] := Abs(cf[j]);
            end for;
            for j := 1 to #cf do
              if (cf[j] ne 0) then
                cf_max := Max([cf_max, 1+Floor(Log(10, Numerator(cf[j]))), 
                                       1+Floor(Log(10, Denominator(cf[j])))]);
              end if;
            end for;
          end if;
        end for;
      end for;
      Write(statisticsFile, "Structure constants of the Lie algebra,");
      Write(statisticsFile, "  maximal number of digits in num/denom:");
      Write(statisticsFile, cf_max);
    end if;
    //  ######  end write statistics  ######

  else 
    Lrep := pLrep;
    L := pL;
  end if;


  dsd := DirectSumDecomposition(L);
  if (#dsd ne 2) or ((#dsd eq 2) and (Dimension(dsd[1]) ne 3)) then
    print "The variety is not anticanonically embedded P1xP1";
    print "  - the Lie algebra does not decompose into two 3-dimensional ideals.";
    return [ ];
  end if;

  if Diag then
    x1,y1,h1 := FindChevalleyBasisDiagonal(dsd[1]: statisticsFile := statisticsFile);
  else
    x1,y1,h1 := FindChevalleyBasis(dsd[1]: statisticsFile := statisticsFile);
  end if;
  if (x1 eq 0) then
    print "The variety is not anticanonically embedded P1xP1";
    print "  - the Lie algebra is not split.";
    return [ ];
  end if;
  c:= Eltseq(L ! h1);
  H1 := &+[c[k]*Basis(Lrep)[k] : k in [1..6]];
  c:= Eltseq(L ! y1);
  Y1 := &+[c[k]*Basis(Lrep)[k] : k in [1..6]];
  c:= Eltseq(L ! x1);
  X1 := &+[c[k]*Basis(Lrep)[k] : k in [1..6]];

  if Diag then
    x2,y2,h2 := FindChevalleyBasisDiagonal(dsd[2]: statisticsFile := statisticsFile);
  else
    x2,y2,h2 := FindChevalleyBasis(dsd[2]: statisticsFile := statisticsFile);
  end if;
  if (x2 eq 0) then
    print "The variety is not anticanonically embedded P1xP1";
    print "  - the Lie algebra is not split.";
    return [ ];
  end if;
  c:= Eltseq(L ! h2);
  H2 := &+[c[k]*Basis(Lrep)[k] : k in [1..6]];
  c:= Eltseq(L ! y2);
  Y2 := &+[c[k]*Basis(Lrep)[k] : k in [1..6]];
  c:= Eltseq(L ! x2);
  X2 := &+[c[k]*Basis(Lrep)[k] : k in [1..6]];

  space := Eigenspace(Transpose(H1), 2) meet Eigenspace(Transpose(H2), 2);
  if (Dimension(space) ne 1) then
    print "The variety is not anticanonically embedded P1xP1";
    print "  - the module is not as expected.";
    return [ ];
  end if;
  max_weight_vec := Basis(space)[1];

  tY1 := Transpose(Y1);
  tY2 := Transpose(Y2);

  v1 := max_weight_vec;
  v2 := 1/2 * v1 * tY1;
  v3 := 1/4 * v2 * tY1;
  v4 := 1/2 * v1 * tY2;
  v5 := 1/2 * v4 * tY1;
  v6 := 1/4 * v5 * tY1;
  v7 := 1/4 * v4 * tY2;
  v8 := 1/4 * v5 * tY2;
  v9 := 1/4 * v6 * tY2;

  M := Matrix([v1,v2,v3,v4,v5,v6,v7,v8,v9]);

  Par<s,t,u> := PolynomialRing(RationalField(), 3);
  T := [u^4,     s*u^3,   s^2*u^2,
        t*u^3,   s*t*u^2, s^2*t*u,
        t^2*u^2, s*t^2*u, s^2*t^2];
  //  now the parametrization of sbs is:  T * M

  X := [ Par | ];
  for i := 1 to 9 do
    X[i] := 0;
    for j := 1 to 9 do
      X[i] +:= T[j]*M[j][i];
    end for;
  end for;

  //  check the parametrization
  phi := hom<R -> Par | 
    X[1], X[2], X[3], X[4], X[5], X[6], X[7], X[8], X[9]>;
  ok := true;
  for i := 1 to #Basis(I) do
    if (phi(Basis(I)[i]) ne 0) then
      ok := false;
      break i;
    end if;
  end for;
  if not ok then 
    print "Parametrization not found.";
    print "The variety is probably not anticanonically embedded P1xP1";
    return [ ];
  end if;

  return X;

end intrinsic;


